import { objectWithoutPropertiesLoose as _objectWithoutPropertiesLoose, extends as _extends } from "../../_virtual/_rollupPluginBabelHelpers.js";
import { defineComponent, ref, provide, computed, inject, watchEffect } from "vue";
import { match } from "../../utils/match.esm.js";
import { render, Features } from "../../utils/render.esm.js";
import { Keys } from "../../keyboard.esm.js";
import { useId } from "../../hooks/use-id.esm.js";
import { dom } from "../../utils/dom.esm.js";
import { useOpenClosedProvider, State, useOpenClosed } from "../../internal/open-closed.esm.js";
import { useResolveButtonType } from "../../hooks/use-resolve-button-type.esm.js";
var DisclosureStates;
(function(DisclosureStates2) {
  DisclosureStates2[DisclosureStates2["Open"] = 0] = "Open";
  DisclosureStates2[DisclosureStates2["Closed"] = 1] = "Closed";
})(DisclosureStates || (DisclosureStates = {}));
var DisclosureContext = /* @__PURE__ */ Symbol("DisclosureContext");
function useDisclosureContext(component) {
  var context = inject(DisclosureContext, null);
  if (context === null) {
    var err = new Error("<" + component + " /> is missing a parent <Disclosure /> component.");
    if (Error.captureStackTrace)
      Error.captureStackTrace(err, useDisclosureContext);
    throw err;
  }
  return context;
}
var DisclosurePanelContext = /* @__PURE__ */ Symbol("DisclosurePanelContext");
function useDisclosurePanelContext() {
  return inject(DisclosurePanelContext, null);
}
var Disclosure = /* @__PURE__ */ defineComponent({
  name: "Disclosure",
  props: {
    as: {
      type: [Object, String],
      "default": "template"
    },
    defaultOpen: {
      type: [Boolean],
      "default": false
    }
  },
  setup: function setup(props, _ref) {
    var slots = _ref.slots, attrs = _ref.attrs;
    var buttonId = "headlessui-disclosure-button-" + useId();
    var panelId = "headlessui-disclosure-panel-" + useId();
    var disclosureState = ref(props.defaultOpen ? DisclosureStates.Open : DisclosureStates.Closed);
    var panelRef = ref(null);
    var buttonRef = ref(null);
    var api = {
      buttonId,
      panelId,
      disclosureState,
      panel: panelRef,
      button: buttonRef,
      toggleDisclosure: function toggleDisclosure() {
        var _match;
        disclosureState.value = match(disclosureState.value, (_match = {}, _match[DisclosureStates.Open] = DisclosureStates.Closed, _match[DisclosureStates.Closed] = DisclosureStates.Open, _match));
      },
      closeDisclosure: function closeDisclosure() {
        if (disclosureState.value === DisclosureStates.Closed)
          return;
        disclosureState.value = DisclosureStates.Closed;
      },
      close: function close(focusableElement) {
        api.closeDisclosure();
        var restoreElement = function() {
          if (!focusableElement)
            return dom(api.button);
          if (focusableElement instanceof HTMLElement)
            return focusableElement;
          if (focusableElement.value instanceof HTMLElement)
            return dom(focusableElement);
          return dom(api.button);
        }();
        restoreElement == null ? void 0 : restoreElement.focus();
      }
    };
    provide(DisclosureContext, api);
    useOpenClosedProvider(computed(function() {
      var _match2;
      return match(disclosureState.value, (_match2 = {}, _match2[DisclosureStates.Open] = State.Open, _match2[DisclosureStates.Closed] = State.Closed, _match2));
    }));
    return function() {
      var passThroughProps = _objectWithoutPropertiesLoose(props, ["defaultOpen"]);
      var slot = {
        open: disclosureState.value === DisclosureStates.Open,
        close: api.close
      };
      return render({
        props: passThroughProps,
        slot,
        slots,
        attrs,
        name: "Disclosure"
      });
    };
  }
});
var DisclosureButton = /* @__PURE__ */ defineComponent({
  name: "DisclosureButton",
  props: {
    as: {
      type: [Object, String],
      "default": "button"
    },
    disabled: {
      type: [Boolean],
      "default": false
    }
  },
  render: function render$1() {
    var api = useDisclosureContext("DisclosureButton");
    var slot = {
      open: api.disclosureState.value === DisclosureStates.Open
    };
    var propsWeControl = this.isWithinPanel ? {
      ref: "el",
      type: this.type,
      onClick: this.handleClick,
      onKeydown: this.handleKeyDown
    } : {
      id: this.id,
      ref: "el",
      type: this.type,
      "aria-expanded": this.$props.disabled ? void 0 : api.disclosureState.value === DisclosureStates.Open,
      "aria-controls": dom(api.panel) ? api.panelId : void 0,
      disabled: this.$props.disabled ? true : void 0,
      onClick: this.handleClick,
      onKeydown: this.handleKeyDown,
      onKeyup: this.handleKeyUp
    };
    return render({
      props: _extends({}, this.$props, propsWeControl),
      slot,
      attrs: this.$attrs,
      slots: this.$slots,
      name: "DisclosureButton"
    });
  },
  setup: function setup2(props, _ref2) {
    var attrs = _ref2.attrs;
    var api = useDisclosureContext("DisclosureButton");
    var panelContext = useDisclosurePanelContext();
    var isWithinPanel = panelContext === null ? false : panelContext === api.panelId;
    var elementRef = ref(null);
    if (!isWithinPanel) {
      watchEffect(function() {
        api.button.value = elementRef.value;
      });
    }
    return {
      isWithinPanel,
      id: api.buttonId,
      el: elementRef,
      type: useResolveButtonType(computed(function() {
        return {
          as: props.as,
          type: attrs.type
        };
      }), elementRef),
      handleClick: function handleClick() {
        if (props.disabled)
          return;
        if (isWithinPanel) {
          var _dom;
          api.toggleDisclosure();
          (_dom = dom(api.button)) == null ? void 0 : _dom.focus();
        } else {
          api.toggleDisclosure();
        }
      },
      handleKeyDown: function handleKeyDown(event) {
        var _dom2;
        if (props.disabled)
          return;
        if (isWithinPanel) {
          switch (event.key) {
            case Keys.Space:
            case Keys.Enter:
              event.preventDefault();
              event.stopPropagation();
              api.toggleDisclosure();
              (_dom2 = dom(api.button)) == null ? void 0 : _dom2.focus();
              break;
          }
        } else {
          switch (event.key) {
            case Keys.Space:
            case Keys.Enter:
              event.preventDefault();
              event.stopPropagation();
              api.toggleDisclosure();
              break;
          }
        }
      },
      handleKeyUp: function handleKeyUp(event) {
        switch (event.key) {
          case Keys.Space:
            event.preventDefault();
            break;
        }
      }
    };
  }
});
var DisclosurePanel = /* @__PURE__ */ defineComponent({
  name: "DisclosurePanel",
  props: {
    as: {
      type: [Object, String],
      "default": "div"
    },
    "static": {
      type: Boolean,
      "default": false
    },
    unmount: {
      type: Boolean,
      "default": true
    }
  },
  render: function render$12() {
    var api = useDisclosureContext("DisclosurePanel");
    var slot = {
      open: api.disclosureState.value === DisclosureStates.Open,
      close: api.close
    };
    var propsWeControl = {
      id: this.id,
      ref: "el"
    };
    return render({
      props: _extends({}, this.$props, propsWeControl),
      slot,
      attrs: this.$attrs,
      slots: this.$slots,
      features: Features.RenderStrategy | Features.Static,
      visible: this.visible,
      name: "DisclosurePanel"
    });
  },
  setup: function setup3() {
    var api = useDisclosureContext("DisclosurePanel");
    provide(DisclosurePanelContext, api.panelId);
    var usesOpenClosedState = useOpenClosed();
    var visible = computed(function() {
      if (usesOpenClosedState !== null) {
        return usesOpenClosedState.value === State.Open;
      }
      return api.disclosureState.value === DisclosureStates.Open;
    });
    return {
      id: api.panelId,
      el: api.panel,
      visible
    };
  }
});
export { Disclosure, DisclosureButton, DisclosurePanel };
//# sourceMappingURL=disclosure.esm.js.map
